/*
 * EMR7370.c
 *
 * Created: 08.04.2012 14:53:52
 *  Author: Chris
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include "RFM01.h"
#include "EMR7370.h"

t_emr emr_data;

void EMR_Init() {
	RFM_DDR = RFM_SCK | RFM_NFFS | RFM_NSEL| RFM_MOSI;
	RFM_PORT |= RFM_NFFS;
	
	RFM_Deselect();
	_delay_ms(3);
	RFM_Select();
	_delay_ms(5);
	
	//configuration setting command
	RFM_WriteConf(RFM_CMD_CFG_SET | RFM_CFG_FRQ_868 | RFM_CFG_BBW_200 | RFM_CFG_CAP_15PF | RFM_CFG_CLK_OFF);
	
	//Frequency setting Command
	RFM_WriteConf(RFM_CMD_FRQ | RFM_FRQ_868); 
	
	//Receiver Setting Command, receiver disabled
	RFM_WriteConf(RFM_CMD_RXS | RFM_RXS_VDI_CRL); 
	
	//Receiver Setting Command, receiver enabled
	RFM_WriteConf(RFM_CMD_RXS | RFM_RXS_VDI_CRL | RFM_RXS_EN);
	
	_delay_ms(500);
	
	//Output and FIFO mode command
	RFM_WriteConf(RFM_CMD_FIFO | RFM_FIFO_IT_LVL | RFM_FIFO_ST_SYNC | RFM_FIFO_DEEP);
	
	//Output and FIFO mode command
	RFM_WriteConf(RFM_CMD_FIFO | RFM_FIFO_IT_LVL | RFM_FIFO_ST_SYNC | RFM_FIFO_DEEP | RFM_FIFO_ENABLE); 
	
	//Data Filter Command
	RFM_WriteConf(RFM_CMD_FLT | RFM_FLT_CLKREC | RFM_FLT_TYP_DIG | RFM_FLT_DQD_THR);
	
	//Data Rate Command
	RFM_WriteConf(RFM_CMD_DR | 0x23);
	
	//AFC command
	RFM_WriteConf(RFM_CMD_AFC | RFM_AFC_AOM_IND | RFM_AFC_RL_0 | RFM_AFC_FI | RFM_AFC_OE | RFM_AFC_EN);
	
	
	//RFM_WriteConf(0xB014); //unknown command
	
	//Low Battery Detector and C Clock Divider Command
	RFM_WriteConf(RFM_CMD_LBCD | RFM_LBCD_1MHZ | RFM_LBCD_VLB_275);
	
	
	//RFM_WriteConf(0xC080); //Receiver Setting Command, receiver disabled
	RFM_WriteConf(RFM_CMD_RXS | RFM_RXS_VDI_CRL);
	
	_delay_ms(188);
	
	//RFM_WriteConf_fast(0xC081); //Receiver Setting Command, receiver enabled
	RFM_WriteConf_fast(RFM_CMD_RXS | RFM_RXS_VDI_CRL | RFM_RXS_EN);
						
	_delay_ms(4);		
						
	//RFM_WriteConf_fast(0x94D9); //configuration setting command
	RFM_WriteConf_fast(RFM_CMD_CFG_SET | RFM_CFG_FRQ_868 | RFM_CFG_EB | RFM_CFG_CAP_15PF | RFM_CFG_BBW_200 | RFM_CFG_CLK_OFF);
	_delay_us(234);
	
	//Low Battery Detector and C Clock Divider Command
	//RFM_WriteConf_fast(RFM_CMD_LBCD | RFM_LBCD_VLB_315);
	
	_delay_ms(5);
	
	//not essentially needed
	/*RFM_Select();	
	RFM_WriteByte_fast(0);
	RFM_Deselect();*/
	
	//configuration setting command
	RFM_WriteConf_fast(RFM_CMD_CFG_SET | RFM_CFG_FRQ_868 | RFM_CFG_BBW_200 | RFM_CFG_CAP_15PF | RFM_CFG_CLK_OFF);
	_delay_us(178);
	//Receiver Setting Command, receiver disabled
	RFM_WriteConf_fast(RFM_CMD_RXS | RFM_RXS_VDI_CRL);
	
	
	RFM_IRQ_Enable();
	
	_delay_ms(52);
	
	//RFM_WriteConf_fast(0xC081); //Receiver Setting Command, receiver enabled
	RFM_WriteConf_fast(RFM_CMD_RXS | RFM_RXS_VDI_CRL | RFM_RXS_EN);
}

uint8_t CalcCrc8(const unsigned char *Buffer, uint8_t Offset, uint8_t len) {
	#define CRC8_POLYNOM 0x7F
	uint8_t crc = CRC8_POLYNOM;
	while(len) {
		len--;
		crc -= Buffer[Offset++];
		crc &= CRC8_POLYNOM;
	}
	return crc;
}

uint8_t EMR_DecodePacket() {
	//check constant bytes
	if(rfm_data[0] != 0x25) return 1;
	if(rfm_data[1] != 0x6A) return 1;
	/*if(rfm_data[4] != 0x40) return 1;
	if(rfm_data[9] != 0x40) return 1;*/
	
	if(CalcCrc8((uint8_t *)&rfm_data, 0, 12) != 127) return 2;
	
	emr_data.address = rfm_data[2] << 8 | rfm_data[3];
	emr_data.power   = (rfm_data[4] & 0x3F) << 8 | rfm_data[5];
	emr_data.current = rfm_data[6] << 8 | rfm_data[7];
	emr_data.voltage = rfm_data[8];
	emr_data.energy  = (rfm_data[9] & 0x3F) << 8 | rfm_data[10];
	
	return 0;
}

void EMR_StartRx() {
	//RFM_WriteConf_fast(0xC081);
	RFM_WriteConf_fast(RFM_CMD_RXS | RFM_RXS_VDI_CRL | RFM_RXS_EN);
	_delay_ms(3);
	//RFM_WriteConf_fast(0xCE85);
	RFM_WriteConf_fast(RFM_CMD_FIFO | RFM_FIFO_IT_LVL | RFM_FIFO_ST_SYNC | RFM_FIFO_DEEP); //Output and FIFO mode command
	_delay_us(170);
	//RFM_WriteConf_fast(0xCE87);
	RFM_WriteConf_fast(RFM_CMD_FIFO | RFM_FIFO_IT_LVL | RFM_FIFO_ST_SYNC | RFM_FIFO_DEEP | RFM_FIFO_ENABLE); //Output and FIFO mode command
	rfm_bpos = 0;
}			