/*
 * RFM01.c
 *
 * Author: chris (at) hobbyelektronik (dot) org
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "RFM01.h"

volatile uint8_t rfm_bpos = 0;
volatile uint8_t rfm_data[RFM_PACKETLENGTH];

void RFM_WriteByte_fast(uint8_t data) {
	for(uint8_t i = 0; i < 8; i++) {
		if(data & 0x80) {
			RFM_PORT |= RFM_MOSI;
		} else {
			RFM_PORT &= ~RFM_MOSI;
		}
		data <<= 1;
		nop();
		
		RFM_PORT |= RFM_SCK;
		_delay_us(2);
		RFM_PORT &= ~RFM_SCK;
		_delay_us(8);
	}
}

void RFM_WriteConf(uint16_t data) {
	RFM_Select();
	_delay_ms(4);
	
	for(uint8_t i = 0; i < 16; i++) {
		if(data & 0x8000) {
			RFM_PORT |= RFM_MOSI;
		} else {
			RFM_PORT &= ~RFM_MOSI;
		}
		data <<= 1;
		nop();
		
		RFM_PORT |= RFM_SCK;
		_delay_us(550);
		RFM_PORT &= ~RFM_SCK;
		_delay_ms(4);
	}
	
	RFM_Deselect();
	_delay_ms(3);
}

void RFM_WriteConf_fast(uint16_t data) {
	RFM_Select();
	_delay_us(38);
	
	RFM_WriteByte_fast(data >> 8);
	RFM_WriteByte_fast(data & 0xFF);
	
	
	RFM_Deselect();
	_delay_us(20);
}


uint8_t RFM_ReadData(uint8_t length) {
	uint8_t data = 0;
	for(uint8_t i = 0; i < length; i++) {
		RFM_PORT |= RFM_SCK;
		_delay_us(2);
		data <<= 1;
		if(RFM_PIN & RFM_MISO) {
			data |= 1;
		}
		nop();
		
		RFM_PORT &= ~RFM_SCK;
		_delay_us(8);
	}
	return data;
}

ISR(INT0_vect) {
	RFM_Select();
	_delay_us(8);
	RFM_ReadData(4);
	
	RFM_Deselect();
	_delay_us(8);
	RFM_Select();
	RFM_PORT &= ~RFM_NFFS;
	
	rfm_data[rfm_bpos] = RFM_ReadData(8);
	RFM_PORT |= RFM_NFFS;	
	
	RFM_Deselect();
	
	rfm_bpos++;
	
	if(rfm_bpos == RFM_PACKETLENGTH) {
		//RFM_WriteConf_fast(0xCE85); //reenable fifo
		RFM_WriteConf_fast(RFM_CMD_FIFO | RFM_FIFO_IT_LVL | RFM_FIFO_ST_SYNC | RFM_FIFO_DEEP); //Output and FIFO mode command
		//RFM_WriteConf_fast(0xC080); //receiver disabled
		RFM_WriteConf_fast(RFM_CMD_RXS | RFM_RXS_VDI_CRL);
	}
}

void RFM_IRQ_Enable() {
	MCUCR |= (1<<ISC01); // The falling edge of INT0 generates an interrupt request
	GICR |= (1<<INT0); // External Interrupt Request 0 Enable
}